<?php
/*
* Define class wwcAmzAffStatsProd
* Make sure you skip down to the end of this file, as there are a few
* lines of code that are very important.
*/
!defined('ABSPATH') and exit;
      
if (class_exists('wwcAmzAffStatsProd') != true) {
    class wwcAmzAffStatsProd
    {
        /*
        * Some required plugin information
        */
        const VERSION = '1.0';

        /*
        * Store some helpers config
        */
		public $the_plugin = null;

		private $module_folder = '';
		private $module = '';

		static protected $_instance;

        /*
        * Required __construct() function that initalizes the AA-Team Framework
        */
        public function __construct()
        {
        	global $wwcAmzAff;

        	$this->the_plugin = $wwcAmzAff;
			$this->module_folder = $this->the_plugin->cfg['paths']['plugin_dir_url'] . 'modules/stats_prod/';
			$this->module = $this->the_plugin->cfg['modules']['stats_prod'];
   
			if (is_admin()) {
	            add_action('admin_menu', array( &$this, 'adminMenu' ));
			}

			if ( $this->the_plugin->is_admin !== true ) {
				$this->addFrontFilters();
			}
        }

		/**
	    * Singleton pattern
	    *
	    * @return wwcAmzAffStatsProd Singleton instance
	    */
	    static public function getInstance()
	    {
	        if (!self::$_instance) {
	            self::$_instance = new self;
	        }

	        return self::$_instance;
	    }

		/**
	    * Hooks
	    */
	    static public function adminMenu()
	    {
	       self::getInstance()
	    		->_registerAdminPages();
	    }

	    /**
	    * Register plug-in module admin pages and menus
	    */
		protected function _registerAdminPages()
    	{
    		add_submenu_page(
    			$this->the_plugin->alias,
    			$this->the_plugin->alias . " " . __('Products Stats', $this->the_plugin->localizationName),
	            __('Products Stats', $this->the_plugin->localizationName),
	            'manage_options',
	            $this->the_plugin->alias . "_stats_prod",
	            array($this, 'display_index_page')
	        );

			return $this;
		}

		public function display_index_page()
		{
			$this->printBaseInterface();
		}
		
		/**
		 * frontend methods: update hits & add to cart for amazon product!
		 *
		 */
		public function addFrontFilters() {
			add_action('wp', array( $this, 'frontend' ), 0);
			
			add_action('woocommerce_add_to_cart', array($this, 'add_to_cart'), 1, 6); // add item to cart
			add_action('wp_ajax_woocommerce_add_to_cart', array($this, 'add_to_cart_ajax'), 0);
			add_action('wp_ajax_nopriv_woocommerce_add_to_cart', array($this, 'add_to_cart_ajax'), 0);
		}

		public function frontend() {
			global $wpdb, $wp;

			// $currentUri = home_url(add_query_arg(array(), $wp->request));

			if ( !is_admin() /*&& is_singular()*/ ) {
				global $post;
				$post_id = (int)$post->ID;
  
				// verify if it's an woocommerce amazon product!
				if ( $post_id <= 0 || !$this->the_plugin->verify_product_isamazon($post_id) )
					return false;

				// update hits
				$hits = (int) get_post_meta($post_id, '_amzaff_hits', true);
				update_post_meta($post_id, '_amzaff_hits', (int)($hits+1));
			}
		}

		public function add_to_cart_validation( $passed, $product_id, $quantity, $variation_id='', $variations='' ) {
			if ( !is_admin() ) {
				$post_id = $product_id;

				// verify if it's an woocommerce amazon product!
				if ( $post_id <= 0 || !$this->the_plugin->verify_product_isamazon($post_id) )
					return false;
				
				$addtocart = (int) get_post_meta($post_id, '_amzaff_addtocart', true);
				update_post_meta($post_id, '_amzaff_addtocart', (int)($addtocart+1));
				
				return true;
			}
		}
		
		public function add_to_cart_ajax() {
			global $woocommerce;
  
			check_ajax_referer( 'add-to-cart', 'security' );
			
			$product_id = (int) apply_filters('woocommerce_add_to_cart_product_id', $_POST['product_id']);
			
			$passed_validation = apply_filters('woocommerce_add_to_cart_validation', true, $product_id, 1);
			
			if ($passed_validation && $woocommerce->cart->add_to_cart($product_id, 1)) :
				// Return html fragments
				$data = apply_filters('add_to_cart_fragments', array());
				
				$post_id = $product_id;

				// verify if it's an woocommerce amazon product!
				if ( $post_id <= 0 || !$this->the_plugin->verify_product_isamazon($post_id) )
					return false;
				
				$addtocart = (int) get_post_meta($post_id, '_amzaff_addtocart', true);
				update_post_meta($post_id, '_amzaff_addtocart', (int)($addtocart+1));
			else :
				// If there was an error adding to the cart, redirect to the product page to show any errors
				$data = array(
					'error' => true,
					'product_url' => get_permalink( $product_id )
				);
				$woocommerce->set_messages();
			endif;
			
			echo json_encode( $data );
			
			die();
		}
		
		public function add_to_cart( $cart_item_key='', $product_id='', $quantity='', $variation_id='', $variation='', $cart_item_data='' ) {
  
			if ( !is_admin() ) {
				$post_id = $product_id;

				// verify if it's an woocommerce amazon product!
				if ( $post_id <= 0 || !$this->the_plugin->verify_product_isamazon($post_id) )
					return false;
				
				$addtocart = (int) get_post_meta($post_id, '_amzaff_addtocart', true);
				update_post_meta($post_id, '_amzaff_addtocart', (int)($addtocart+1));

				return true;
			}
		}
		

		/*
		* printBaseInterface, method
		* --------------------------
		*
		* this will add the base DOM code for you options interface
		*/
		private function printBaseInterface()
		{
			global $wpdb;
?>
		<script type="text/javascript" src="<?php echo $this->module_folder;?>app.class.js" ></script>
		<div id="wwcAmzAff-wrapper" class="fluid wrapper-wwcAmzAff">
			
			<?php
			// show the top menu
			wwcAmzAffAdminMenu::getInstance()->make_active('info|stats_prod')->show_menu(); 
			?>

			<!-- Content -->
			<div id="wwcAmzAff-content">
				
				<h1 class="wwcAmzAff-section-headline">
					<?php 
					if( isset($this->module['stats_prod']['in_dashboard']['icon']) ){
						echo '<img src="' . ( $this->module_folder . $this->module['stats_prod']['in_dashboard']['icon'] ) . '" class="wwcAmzAff-headline-icon">';
					}
					?>
					<?php echo $this->module['stats_prod']['menu']['title'];?>
					<span class="wwcAmzAff-section-info"><?php echo $this->module['stats_prod']['description'];?></span>
					<?php
					$has_help = isset($this->module['stats_prod']['help']) ? true : false;
					if( $has_help === true ){
						
						$help_type = isset($this->module['stats_prod']['help']['type']) && $this->module['stats_prod']['help']['type'] ? 'remote' : 'local';
						if( $help_type == 'remote' ){
							echo '<a href="#load_docs" class="wwcAmzAff-show-docs" data-helptype="' . ( $help_type ) . '" data-url="' . ( $this->module['stats_prod']['help']['url'] ) . '">HELP</a>';
						} 
					}
					echo '<a href="#load_docs" class="wwcAmzAff-show-feedback" data-helptype="' . ( 'remote' ) . '" data-url="' . ( $this->the_plugin->feedback_url ) . '" data-operation="feedback">Feedback</a>';
					?>
				</h1>
				
				<!-- Main loading box -->
				<div id="wwcAmzAff-main-loading">
					<div id="wwcAmzAff-loading-overlay"></div>
					<div id="wwcAmzAff-loading-box">
						<div class="wwcAmzAff-loading-text"><?php _e('Loading', $this->the_plugin->localizationName);?></div>
						<div class="wwcAmzAff-meter wwcAmzAff-animate" style="width:86%; margin: 34px 0px 0px 7%;"><span style="width:100%"></span></div>
					</div>
				</div>

				<!-- Container -->
				<div class="wwcAmzAff-container clearfix">

					<!-- Main Content Wrapper -->
					<div id="wwcAmzAff-content-wrap" class="clearfix">

						<!-- Content Area -->
						<div id="wwcAmzAff-content-area">
							<div class="wwcAmzAff-grid_4">
	                        	<div class="wwcAmzAff-panel">
	                        		<!--div class="wwcAmzAff-panel-header">
										<span class="wwcAmzAff-panel-title">
											<?php _e('Synchronization logs', $this->the_plugin->localizationName);?>
										</span>
									</div-->
									<div class="wwcAmzAff-panel-content">
										<form class="wwcAmzAff-form" action="#save_with_ajax">
											<div class="wwcAmzAff-form-row wwcAmzAff-table-ajax-list" id="wwcAmzAff-table-ajax-response">
											<?php
											wwcAmzAffAjaxListTable::getInstance( $this->the_plugin )
												->setup(array(
													'id' 				=> 'wwcAmzAffPriceUpdateMonitor',
													'show_header' 		=> true,
													'search_box' 		=> false,
													'items_per_page' 	=> '15',
													'post_statuses' 	=> array(
														'publish'   => __('Published', $this->the_plugin->localizationName)
													),
													'list_post_types'	=> array('product'),
													'columns'			=> array(

														'id'		=> array(
															'th'	=> __('ID', $this->the_plugin->localizationName),
															'td'	=> '%ID%',
															'width' => '40'
														),
														
														'thumb'		=> array(
															'th'	=> __('Thumb', $this->the_plugin->localizationName),
															'td'	=> '%thumb%',
															'align' => 'center',
															'width' => '50'
														),
														
														'asin'		=> array(
															'th'	=> __('ASIN', $this->the_plugin->localizationName),
															'td'	=> '%asin%',
															'align' => 'center',
															'width' => '70'
														),

														'title'		=> array(
															'th'	=> __('Title', $this->the_plugin->localizationName),
															'td'	=> '%title%',
															'align' => 'left'
														),
														/*
														'price'		=> array(
															'th'	=> __('Price', $this->the_plugin->localizationName),
															'td'	=> '%price%',
															'align' => 'left',
															'width' => '160'
														),*/
														
														'hits'		=> array(
															'th'	=> __('Hits', $this->the_plugin->localizationName),
															'td'	=> '%hits%',
															'align' => 'center',
															'width' => '70'
														),
														
														'added_to_cart'		=> array(
															'th'	=> __('Added to cart', $this->the_plugin->localizationName),
															'td'	=> '%added_to_cart%',
															'align' => 'center',
															'width' => '90'
														),
														
														'redirected_to_amazon'	=> array(
															'th'	=> __('Redirected to Amazon', $this->the_plugin->localizationName),
															'td'	=> '%redirected_to_amazon%',
															'align' => 'center',
															'width' => '140'
														),
														
														'date'		=> array(
															'th'	=> __('Date Added', $this->the_plugin->localizationName),
															'td'	=> '%date%',
															'width' => '120'
														),

														/*'optimize_btn' => array(
															'th'	=> __('Action', $this->the_plugin->localizationName),
															'td'	=> '%button%',
															'option' => array(
																'value' => __('Update product now', $this->the_plugin->localizationName),
																'action' => 'sync_now',
																'color' => 'blue'
															),
															'width' => '120'
														),*/
													)
												))
												->print_html();
								            ?>
								            </div>
							            </form>
				            		</div>
								</div>
							</div>
							<div class="clear"></div>
						</div>
					</div>
				</div>
			</div>
		</div>

<?php
		}
    }
}
 
// Initialize the wwcAmzAffStatsProd class
$wwcAmzAffStatsProd = wwcAmzAffStatsProd::getInstance();
