<?php
/*
* Define class wwcAmzAffTailSyncMonitor
* Make sure you skip down to the end of this file, as there are a few
* lines of code that are very important.
*/
!defined('ABSPATH') and exit;
if (class_exists('wwcAmzAffTailSyncMonitor') != true) {
    class wwcAmzAffTailSyncMonitor
    {
        /*
        * Some required plugin information
        */
        const VERSION = '1.0';

        /*
        * Store some helpers config
        */
		public $the_plugin = null;
        private $alias = '';

		private $module_folder = '';
		private $module = '';

		static protected $_instance;
        
        private static $sql_chunk_limit = 2000;
        
        //<span>Price</span><span>Title</span><span>Buy URL</span><span>Description</span><span>SKU</span><span>Sales Rank</span>
        static protected $sync_fields = array();
        static protected $sync_recurrence = array();
        static protected $sync_hour_start = array();
        static protected $sync_products_per_request = array();
        
        static private $settings = array();

        /*
        * Required __construct() function that initalizes the AA-Team Framework
        */
        public function __construct()
        {
        	global $wwcAmzAff;

        	$this->the_plugin = $wwcAmzAff;
            $this->alias = $this->the_plugin->alias;
			$this->module_folder = $this->the_plugin->cfg['paths']['plugin_dir_url'] . 'modules/synchronization/';
            $this->module_folder_path = $this->the_plugin->cfg['paths']['plugin_dir_path'] . 'modules/synchronization/';
			$this->module = $this->the_plugin->cfg['modules']['synchronization'];
            
            $ss = get_option($this->alias . '_sync', array());
            $ss = maybe_unserialize($ss);
            self::$settings = $ss !== false ? $ss : array();

            // sync options
            self::$sync_fields = array(
                'price'                 => __('Price', $this->the_plugin->localizationName),
                'title'                 => __('Title', $this->the_plugin->localizationName),
                'url'                   => __('Buy URL', $this->the_plugin->localizationName),
                'desc'                  => __('Description', $this->the_plugin->localizationName),
                'sku'                   => __('SKU', $this->the_plugin->localizationName),
                'sales_rank'            => __('Sales Rank', $this->the_plugin->localizationName),
            );
            self::$sync_recurrence = array(
                24      => __('Every single day', $this->the_plugin->localizationName),
                48      => __('Every 2 days', $this->the_plugin->localizationName),
                72      => __('Every 3 days', $this->the_plugin->localizationName),
                96      => __('Every 4 days', $this->the_plugin->localizationName),
                120     => __('Every 5 days', $this->the_plugin->localizationName),
                144     => __('Every 6 days', $this->the_plugin->localizationName),
                168     => __('Every 1 week', $this->the_plugin->localizationName),
                336     => __('Every 2 weeks', $this->the_plugin->localizationName),
                504     => __('Every 3 weeks', $this->the_plugin->localizationName),
                720     => __('Every 1 month', $this->the_plugin->localizationName), // ~ 4 weeks + 2 days
            );
            self::$sync_hour_start = $this->the_plugin->doRange( range(0, 23) );
            self::$sync_products_per_request = $this->the_plugin->doRange( range(5, 50, 5) );

			if (is_admin()) {
	            add_action('admin_menu', array( &$this, 'adminMenu' ));
			}

			// ajax helper
			add_action('wp_ajax_wwcAmzAffSyncAjax', array( &$this, 'ajax_request' ));
        }

		/**
	    * Singleton pattern
	    *
	    * @return wwcAmzAffTailSyncMonitor Singleton instance
	    */
	    static public function getInstance()
	    {
	        if (!self::$_instance) {
	            self::$_instance = new self;
	        }

	        return self::$_instance;
	    }

		/**
	    * Hooks
	    */
	    static public function adminMenu()
	    {
	       self::getInstance()
	    		->_registerAdminPages();
	    }

	    /**
	    * Register plug-in module admin pages and menus
	    */
		protected function _registerAdminPages()
    	{ 
    		add_submenu_page(
    			$this->the_plugin->alias,
    			$this->the_plugin->alias . " " . __('Synchronization log', $this->the_plugin->localizationName),
	            __('Synchronization log'),
	            'manage_options',
	            $this->the_plugin->alias . "_synclog",
	            array($this, 'display_index_page')
	        );

			return $this;
		}

		public function display_index_page()
		{
			$this->printBaseInterface();
		}
		

		/*
		* printBaseInterface, method
		* --------------------------
		*
		* this will add the base DOM code for you options interface
		*/
		public function printBaseInterface( $module='synchronization' ) {
			global $wpdb;
            
            $ss = self::$settings;

            $mod_vars = array();

            // Sync
            $mod_vars['mod_menu'] = 'info|synchronization_log';
            $mod_vars['mod_title'] = __('Synchronization logs', $this->the_plugin->localizationName);

            // Products Stats
            if ( $module == 'stats_prod' ) {
                $mod_vars['mod_menu'] = 'info|stats_prod';
                $mod_vars['mod_title'] = __('Products stats', $this->the_plugin->localizationName);
            }
            extract($mod_vars);
            
            $module_data = $this->the_plugin->cfg['modules']["$module"];
            $module_folder = $this->the_plugin->cfg['paths']['plugin_dir_url'] . "modules/$module/";
?>
		<script type="text/javascript" src="<?php echo $this->module_folder;?>app.class.js" ></script>
		<script type="text/javascript" src="<?php echo $this->module_folder;?>assets/jquery.tipsy.js" ></script>
		
		<link rel='stylesheet' href='<?php echo $this->module_folder;?>assets/sync-log.css' type='text/css' media='all' />
		<link rel='stylesheet' href='<?php echo $this->module_folder;?>assets/tipsy.css' type='text/css' media='all' />
		
		<div id="wwcAmzAff-wrapper" class="fluid wrapper-wwcAmzAff">
			
			<?php
			// show the top menu
			wwcAmzAffAdminMenu::getInstance()->make_active($mod_menu)->show_menu(); 
			?>
			
			<!-- Content -->
			<div id="wwcAmzAff-content">
			    
				<h1 class="wwcAmzAff-section-headline">
					<?php 
					if( isset($module_data["$module"]['in_dashboard']['icon']) ){
						echo '<img src="' . ( $module_folder . $module_data["$module"]['in_dashboard']['icon'] ) . '" class="wwcAmzAff-headline-icon">';
					}
					?>
					<?php echo $module_data["$module"]['menu']['title'];?>
					<span class="wwcAmzAff-section-info"><?php echo $module_data["$module"]['description'];?></span>
					<?php
					$has_help = isset($module_data["$module"]['help']) ? true : false;
					if( $has_help === true ){
						
						$help_type = isset($module_data["$module"]['help']['type']) && $module_data["$module"]['help']['type'] ? 'remote' : 'local';
						if( $help_type == 'remote' ){
							echo '<a href="#load_docs" class="wwcAmzAff-show-docs" data-helptype="' . ( $help_type ) . '" data-url="' . ( $module_data["$module"]['help']['url2'] ) . '">HELP</a>';
						} 
					}
					echo '<a href="#load_docs" class="wwcAmzAff-show-feedback" data-helptype="' . ( 'remote' ) . '" data-url="' . ( $this->the_plugin->feedback_url ) . '" data-operation="feedback">Feedback</a>';
					?>
				</h1>

				<!-- Container -->
				<div class="wwcAmzAff-container clearfix">

					<!-- Main Content Wrapper -->
					<div id="wwcAmzAff-content-wrap" class="clearfix" style="padding-top: 20px;">

						<!-- Content Area -->
						<div id="wwcAmzAff-content-area">
							<div class="wwcAmzAff-grid_4">
	                        	<div class="wwcAmzAff-panel">
	                        		<div class="wwcAmzAff-panel-header">
										<span class="wwcAmzAff-panel-title"><?php echo $mod_title; ?></span>
										<span class="wwcAmzAff-panel-sync-all"></span>
									</div>
									<div id="wwcAmzAff-sync-log" class="wwcAmzAff-panel-content" data-module="<?php echo $module; ?>">

									    <?php
									       $lang = array(
									           'no_products'          => __('No products available.', 'wwcAmzAff'),
									           'sync_now'             => __('Sync now (<span>{nb}</span> remained)', 'wwcAmzAff'),
									           'sync_now_msgformat'   => __('(ASIN: {1} / ID: #{2}): {3}', 'wwcAmzAff'),
									           'sync_now_finished'    => __('Sync now is finished.', 'wwcAmzAff'),
									           'sync_now_inwork'      => __('Sync all now in progress. Please be patient till it\'s finished.', 'wwcAmzAff'),
									           'sync_now_stop_btn'    => __('stop processing.', 'wwcAmzAff'),
									           'sync_now_stopped'     => __('Sync now is stopped.', 'wwcAmzAff'),
									           'loading'              => __('Loading.', 'wwcAmzAff'),
                                           );
									    ?>
									    <div id="wwcAmzAff-lang-translation" style="display: none;"><?php echo htmlentities(json_encode( $lang )); ?></div>

                                        <!-- Main loading box -->
                                        <div id="wwcAmzAff-main-loading">
                                            <div id="wwcAmzAff-loading-overlay"></div>
                                            <div id="wwcAmzAff-loading-box">
                                                <div class="wwcAmzAff-loading-text"><?php _e('Loading', $this->the_plugin->localizationName);?></div>
                                                <div class="wwcAmzAff-meter wwcAmzAff-animate" style="width:86%; margin: 34px 0px 0px 7%;"><span style="width:100%"></span></div>
                                            </div>
                                        </div>
            
										<!--<div class="wwcAmzAff-sync-filters">
											<select>
												<option>Show All</option>
												<option>Show None</option>
												<option>Show What</option>
											</select>	
											<select>
												<option>Show All</option>
												<option>Show None</option>
												<option>Show What</option>
											</select>
											<a>Published <span class="count">(27)</span></a>
										</div>-->
										<div class="wwcAmzAff-sync-filters">
    										<span>
    										    <?php _e('Total products', $this->the_plugin->localizationName);?>: <span class="count"></span> (<span class="countv"></span> variations)
    										    <!-- | <?php _e('Synchronized products', $this->the_plugin->localizationName);?>: <span class="count">(27)</span>-->
    										</span>
    										<span class="right">
    										    <?php if ( $module == 'synchronization' ) { ?>
        										    <label for="sync_stop_reload"><?php _e('stop auto reload', $this->the_plugin->localizationName); ?></label>
        										    <input type="checkbox" name="sync_stop_reload" id="sync_stop_reload"<?php echo isset($ss['sync_stop_reload']) && !empty($ss['sync_stop_reload']) ? ' checked="checked"' : ''; ?>/>
                                                    <strong>0</strong> <?php _e('seconds', $this->the_plugin->localizationName); ?>
                                                <?php } ?>
    										    <button class="load_prods"><?php _e('Reload products list', $this->the_plugin->localizationName);?></button>
    										    <?php if ( $module == 'synchronization' ) { ?>
    										    <button class="sync-all"><?php _e('Sync all now', $this->the_plugin->localizationName);?></button>
    										    <?php } ?>
    										</span>
										</div>
										<div class="wwcAmzAff-sync-table <?php echo ( $module == 'synchronization' ? 'synchronization' : 'stats_prod' ); ?>">
										  <table cellspacing="0">
                                            <thead>
    										  	<tr class="wwcAmzAff-sync-table-header">
    										  	    <?php if ( $module == 'synchronization' ) { ?>
    										  		<th style="width:1.83%;"><i class="fa fa-flag" title="<?php _e('SYNC STATUS', $this->the_plugin->localizationName);?>"></i></th>
    										  		<th style="width:18.11%;"><?php _e('Image', $this->the_plugin->localizationName);?></th>
    										  		<th style="width:48.44%;"><?php _e('Title', $this->the_plugin->localizationName);?></th>
    										  		<th style="width:14.83%;"><?php _e('Number of Synchronisations', $this->the_plugin->localizationName);?></th>
    										  		<th style="width:8.19%;" class="wz-uppercase"><?php _e('Last Sync', $this->the_plugin->localizationName);?></th>
    										  		<th style="width:8.46%;"><?php _e('Action', $this->the_plugin->localizationName);?></th>
    										  		<?php } else { ?>
    										  		<th style="width:3%;"><?php _e('ID', $this->the_plugin->localizationName);?></th>
                                                    <th style="width:16%;"><?php _e('Image', $this->the_plugin->localizationName);?></th>
                                                    <th style="width:50%;"><?php _e('Title', $this->the_plugin->localizationName);?></th>
                                                    <th style="width:7%;"><?php _e('Hits', $this->the_plugin->localizationName);?></th>
                                                    <th style="width:7%;"><?php _e('Added to cart', $this->the_plugin->localizationName);?></th>
                                                    <th style="width:7%;" class="wz-uppercase"><?php _e('Redirected to Amazon', $this->the_plugin->localizationName);?></th>
                                                    <th style="width:10%;"><?php _e('Date Added', $this->the_plugin->localizationName);?></th>
    										  		<?php } ?>
    										  	</tr>
										  	</thead>
                                            <tbody>
                                            <?php
                                                //require_once( $this->module_folder_path . '_html.php');
                                            ?>
										  	</tbody>
										  </table>
										</div>
										<?php if ( $module == 'synchronization' ) { ?>
    										<div class="wwcAmzAff-sync-info">
    										  <h3><?php _e('Synchronisation Settings', $this->the_plugin->localizationName);?></h3>
                                              <?php echo $this->sync_settings(); ?>
    										</div>
										<?php } ?>
				            		</div>
								</div>
							</div>
							<div class="clear"></div>
						</div>
					</div>
				</div>
			</div>
		</div>
		<?php /*<script type='text/javascript'>
			jQuery('i').tipsy({gravity: 'w'});
			
			jQuery(document).ready(function($){
				jQuery("tbody a.wz-show-variations").click(function(){
				  jQuery( this ).parents().eq(1).siblings().toggleClass("wz-hide-me");
				});
			});
		</script>*/ ?>

<?php
		}

        private function sync_settings() {
            ob_start();
            
            $ss = self::$settings;
?>
                                        <form id="wwcAmzAff-sync-settings">
                                          <p><?php _e('Each product has to sync the following', $this->the_plugin->localizationName);?>:
                                            <?php
                                            foreach (self::$sync_fields as $key => $val) {
                                                $is_checked = 'checked="checked"';
                                                if ( !isset($ss['sync_fields'])
                                                    || ( isset($ss['sync_fields']) && !in_array($key, $ss['sync_fields']) ) ) {
                                                    $is_checked = '';
                                                }
                                            ?>
                                            <span>
                                            <label for="sync_fields[<?php echo $key; ?>]"><?php echo $val; ?></label>:
                                            <input type="checkbox" id="sync_fields[<?php echo $key; ?>]" name="sync_fields[<?php echo $key; ?>]" <?php echo $is_checked; ?> />
                                            </span>
                                            <?php
                                            }
                                            ?>
                                            <!--<span>Price</span><span>Title</span><span>Buy URL</span><span>Description</span><span>SKU</span><span>Sales Rank</span>-->
                                          </p>
                                          <p>
                                            <!--Recurrence : <span>24h</span>
                                            First start a hour <span>10</span>-->
                                            <span>
                                            <?php _e('Recurrence', $this->the_plugin->localizationName);?>:
                                            </span>
                                            <select id="sync_recurrence" name="sync_recurrence">
                                            <?php
                                            foreach (self::$sync_recurrence as $key => $val) {
                                                $is_checked = '';
                                                if ( isset($ss['sync_recurrence']) && $key == $ss['sync_recurrence'] ) {
                                                    $is_checked = 'selected="selected"';
                                                }
                                            ?>
                                                <option value="<?php echo $key; ?>" <?php echo $is_checked; ?>><?php echo $val; ?></option>
                                            <?php
                                            }
                                            ?>
                                            </select>
                                            
                                            <span>
                                            <?php _e('Products per request', $this->the_plugin->localizationName);?>:
                                            </span>
                                            <select id="sync_products_per_request" name="sync_products_per_request">
                                            <?php
                                            foreach (self::$sync_products_per_request as $key => $val) {
                                                $is_checked = '';
                                                if ( isset($ss['sync_products_per_request']) && $key == $ss['sync_products_per_request'] ) {
                                                    $is_checked = 'selected="selected"';
                                                }
                                            ?>
                                                <option value="<?php echo $key; ?>" <?php echo $is_checked; ?>><?php echo $val; ?></option>
                                            <?php
                                            }
                                            ?>
                                            </select>
                                            
                                            <?php /*
                                            <span>
                                            <?php _e('First start at hour', $this->the_plugin->localizationName);?>:
                                            </span>
                                            <select id="sync_hour_start" name="sync_hour_start">
                                            <?php
                                            foreach (self::$sync_hour_start as $key => $val) {
                                                $is_checked = '';
                                                if ( isset($ss['sync_hour_start']) && $key == $ss['sync_hour_start'] ) {
                                                    $is_checked = 'selected="selected"';
                                                }
                                            ?>
                                                <option value="<?php echo $key; ?>" <?php echo $is_checked; ?>><?php echo $val; ?></option>
                                            <?php
                                            }
                                            ?>
                                            </select>
                                            */ ?>
                                          </p>
                                          <p>
                                              <button><?php _e('Save settings', $this->the_plugin->localizationName);?></button>
                                          </p>
                                        </form>
<?php
            return ob_get_clean();
        }

        private function get_products( $module='synchronization' ) {
            global $wpdb;
            
            // get products (simple or just parents without variations)
            $sql = "SELECT p.ID, p.post_title, p.post_parent, p.post_date FROM $wpdb->posts as p WHERE 1=1 AND p.post_status = 'publish' AND p.post_parent = 0 AND p.post_type = 'product' ORDER BY p.ID ASC;";
            $res = $wpdb->get_results( $sql, OBJECT_K );
            
            // get product variations (only childs, no parents)
            $sql_childs = "SELECT p.ID, p.post_title, p.post_parent, p.post_date FROM $wpdb->posts as p WHERE 1=1 AND p.post_status = 'publish' AND p.post_parent > 0 AND p.post_type = 'product_variation' ORDER BY p.ID ASC;";
            $res_childs = $wpdb->get_results( $sql_childs, OBJECT_K );
            
            //var_dump('<pre>', $sql, $sql_childs, '</pre>'); die('debug...'); 
            if ( empty($res) && empty($res_childs) ) return array();
            
            // array with parents and their associated childrens
            $parent2child = array();
            foreach ($res_childs as $id => $val) {
                $parent = $val->post_parent;
                
                if ( !isset($parent2child["$parent"]) ) {
                    $parent2child["$parent"] = array();
                }
                $parent2child["$parent"]["$id"] = $val; 
            }

            // products IDs
            $prods = array_merge(array(), array_keys($res), array_keys($res_childs));
            $prods = array_unique($prods);

            // get ASINs
            $prods2asin = array();
            foreach (array_chunk($prods, self::$sql_chunk_limit, true) as $current) {

                $currentP = implode(',', array_map(array($this->the_plugin, 'prepareForInList'), $current));
                $sql_getasin = "SELECT pm.post_id, pm.meta_value FROM $wpdb->postmeta as pm WHERE 1=1 AND pm.meta_key = '_amzASIN' AND pm.post_id IN ($currentP) ORDER BY pm.post_id ASC;";
                $res_getasin = $wpdb->get_results( $sql_getasin, OBJECT_K );
                $prods2asin = $prods2asin + $res_getasin; //array_replace($prods2asin, $res_getasin);
            }
            
            if ( $module == 'synchronization' ) {
                $__meta_toget = array('_amzaff_sync_last_date', '_amzaff_sync_hits', '_amzaff_sync_last_status_msg');
            } else {
                $__meta_toget = array('_amzaff_hits', '_amzaff_addtocart', '_amzaff_redirect_to_amazon');
            }
            // get sync last date & sync hits
            foreach ( (array) $__meta_toget as $meta) {
                $prods2meta["$meta"] = array();

                foreach (array_chunk($prods, self::$sql_chunk_limit, true) as $current) {
    
                    $currentP = implode(',', array_map(array($this->the_plugin, 'prepareForInList'), $current));
    
                    $sql_getmeta = "SELECT pm.post_id, pm.meta_value FROM $wpdb->postmeta as pm WHERE 1=1 AND pm.meta_key = '$meta' AND pm.post_id IN ($currentP) ORDER BY pm.post_id ASC;";
                    $res_getmeta = $wpdb->get_results( $sql_getmeta, OBJECT_K );
                    $prods2meta["$meta"] = $prods2meta["$meta"] + $res_getmeta; //array_replace($prods2meta["$meta"], $res_getmeta);
                }
            }
 
            // get Thumbs
            $thumbs = $this->get_thumbs();
 
            // build html table with products rows
            $last_updated_product = (int) get_option('wwcAmzAff_sync_last_updated_product', true);
            $next_updated_product = $this->get_next_product( $last_updated_product );
            $default = array(
                'module'        => $module,
                'last_id'       => $last_updated_product,
                'next_id'       => $next_updated_product,
            );
 
            $ret = array('status' => 'valid', 'html' => array(), 'nb' => 0, 'nbv' => 0);
            $nbprod = 0;
            $nbprodv = 0;
            foreach ($res as $id => $val) {
                
                if ( !isset($prods2asin["$id"]) ) continue 1; // exclude products without ASIN

                $__p = $this->row_build(array_merge($default, array(
                    'id'            => $id,
                    'val'           => $val,
                    'prods2asin'    => $prods2asin,
                    'thumbs'        => $thumbs,
                    'prods2meta'    => $prods2meta,
                )));
                $__p = array_merge($__p, array(
                    'id'            => $id,
                ));
                
                $is_open = ( $default['next_id']->post_type == 'product_variation'
                    && ( $id == (int) $default['next_id']->post_parent ) ? 1 : 0 );
                $childs_btn = '';
   
                // product variations if it has
                $childs_html = array();
                if ( isset($parent2child["$id"]) ) {
                    $childs = $parent2child["$id"];
                    $childs_nb = count($childs);
                    $cc = 0;
                    foreach ($childs as $childId => $childVal) {

                        $__pc = $this->row_build(array_merge($default, array(
                            'id'            => $childId,
                            'val'           => $childVal,
                            'prods2asin'    => $prods2asin,
                            'thumbs'        => $thumbs,
                            'prods2meta'    => $prods2meta,
                            'is_open'       => $is_open,
                        )));
                        $__pc = array_merge($__pc, array(
                            'id'            => $childId,
                            'parent_id'     => $id,
                            'cc'            => $cc,
                            'childs_nb'     => $childs_nb,
                        ));
 
                        $childs_html[] = $this->row_view_html($__pc, true);
                        
                        $cc++;
                    }

                    $childs_btn = '<a href="#" class="wz-show-variations' . ($is_open ? ' sign-minus' : ' sign-plus') . '">(<span>' . ($is_open ? '-' : '+') . '</span>' . count($childs) . ')</a>';
                    $nbprodv += count($childs);
                } // end product variations loop
                
                $__p['childs_btn'] = $childs_btn;
                
                // product
                $ret['html'][] = $this->row_view_html($__p);
                
                if ( isset($childs_html) && !empty($childs_html) ) {
                    $ret['html'][] = implode(PHP_EOL, $childs_html);
                }

                $nbprod++;
            } // end products loop
            
            $ret = array_merge($ret, array(
                'nb'        => $nbprod,
                'nbv'       => $nbprodv,
            ));
            
            return $ret;
        }

        private function get_next_product( $last_id=0 ) {
            global $wpdb;
            
            $sql = "select p.ID, p.post_type, p.post_parent from $wpdb->posts as p left join $wpdb->postmeta as pm on p.ID = pm.post_id where 1=1 and p.ID > $last_id and p.post_type in ('product', 'product_variation') and p.post_status = 'publish' and pm.meta_key = '_amzASIN' and !isnull(pm.meta_value) order by p.ID asc limit 1;";
            $ret = $wpdb->get_row( $sql );
            return is_null($ret) || $ret === false
                ? array(
                    'ID'            => $last_id+1,
                    'post_type'     => 'product',
                    'post_parent'   => 0
                ) : $ret;
        }

        private function row_build( $pms ) {
            extract($pms);

            $title = $val->post_title;
            $asin = isset($prods2asin["$id"]) ? $prods2asin["$id"]->meta_value : __('missing', $this->the_plugin->localizationName);
                
            //$thumb = 'http://ecx.images-amazon.com/images/I/41mVXvLfOtL._SL75_.jpg';
            $thumb = isset($thumbs["$id"]) && !empty($thumbs["$id"]) ? $thumbs["$id"] : $this->get_thumb_src_default();
                
            $link_edit = sprintf( admin_url('post.php?post=%s&action=edit'), $id);
            
            $add_data = $val->post_date;
            $add_data = $this->the_plugin->last_update_date('true', strtotime($add_data));

            if ( $module == 'synchronization' ) {
            // statuses: NOT SYNCED | ALREADY SYNCED | NEXT TO BE SYNCED
            $sst = array(
                'not_synced'            => __('NOT SYNCED', $this->the_plugin->localizationName),
                'already_synced'        => __('ALREADY SYNCED', $this->the_plugin->localizationName),
                'next_to_synced'        => __('NEXT TO BE SYNCED', $this->the_plugin->localizationName),
            );
            $next_id = $next_id->ID;
            $sync_status = array(
                'css'       => ( $id < $next_id ? 'wz-synced' : ( $id == $next_id ? 'wz-next-sync' : '' ) ),
                'text'      => ( $id < $next_id ? $sst['already_synced'] : ( $id == $next_id ? $sst['next_to_synced'] : $sst['not_synced'] ) ),
            );
            if ( $is_open ) {
                $sync_status['css'] .= ' wz-hide-me';
                $sync_status['css'] = trim($sync_status['css']);
            }
            
            $sync_nb = isset($prods2meta['_amzaff_sync_hits']["$id"]) ? $prods2meta['_amzaff_sync_hits']["$id"]->meta_value : 0;

            $sync_data = isset($prods2meta['_amzaff_sync_last_date']["$id"]) ? $prods2meta['_amzaff_sync_last_date']["$id"]->meta_value : '';
            $sync_data = $this->the_plugin->last_update_date('true', $sync_data);
            
            $sync_last_status = isset($prods2meta['_amzaff_sync_last_status_msg']["$id"]) ? strtoupper('last sync status: ' . $prods2meta['_amzaff_sync_last_status_msg']["$id"]->meta_value) : '';

            $ret = compact('module', 'add_data', 'title', 'asin', 'thumb', 'link_edit', 'sync_status', 'sync_nb', 'sync_data', 'sync_last_status');
            } else {
            $sync_status = array(
                'css'       => ( '' ),
                'text'      => ( '' ),
            );
            
            $stats_hits = isset($prods2meta['_amzaff_hits']["$id"]) ? $prods2meta['_amzaff_hits']["$id"]->meta_value : 0;
            $stats_added_to_cart = isset($prods2meta['_amzaff_addtocart']["$id"]) ? $prods2meta['_amzaff_addtocart']["$id"]->meta_value : 0;
            $stats_redirected_to_amazon = isset($prods2meta['_amzaff_redirect_to_amazon']["$id"]) ? $prods2meta['_amzaff_redirect_to_amazon']["$id"]->meta_value : 0;
            
            $ret = compact('module', 'add_data', 'title', 'asin', 'thumb', 'link_edit', 'sync_status', 'stats_hits', 'stats_added_to_cart', 'stats_redirected_to_amazon');
            }
            return $ret;
        }

        private function row_view_html( $row, $is_child=false ) {
            $tr_css = ' ' . $row['sync_status']['css']
                . ($is_child ? ' wz-variation' . ($row['cc'] == 0 ? ' first' : ($row['cc'] == $row['childs_nb'] - 1 ? ' last' : '')) : '');
            $data_parent_id = ($is_child ? ' data-parent_id=' . $row['parent_id'] : '');
            $childs_btn = (!$is_child ? ' ' . $row['childs_btn'] : '');
            
            $text_id = __('ID', $this->the_plugin->localizationName) . ': #';
            $text_asin = __('ASIN', $this->the_plugin->localizationName) . ': ';
            
            if ( $row['module'] == 'synchronization' ) {
            $text_syncs_nb = sprintf( __('<span>%s</span> SYNCS', $this->the_plugin->localizationName), $row['sync_nb'] );
            $text_syncs_last_status = ''; //(!empty($row['sync_last_status']) ? '<i class="fa fa-comment-o" title="' . $row['sync_last_status'] . '"></i>' : '');
            $text_sync_now = __('SYNC NOW', $this->the_plugin->localizationName);
            } else {
            $text_hits = '<i class="wwcAmzAff-prod-stats-number hits">' . ( $row['stats_hits'] ) . '</i>';
            $text_added_to_cart = '<i class="wwcAmzAff-prod-stats-number add-to-cart">' . ( $row['stats_added_to_cart'] ) . '</i>';
            $text_redirected_to_amazon = '<i class="wwcAmzAff-prod-stats-number redirect-to-amazon">' . ( $row['stats_redirected_to_amazon'] ) . '</i>';
            }
            
            if ( $row['module'] == 'synchronization' ) {
            $ret = '
                    <tr class="wwcAmzAff-sync-table-row' . $tr_css . '" data-id=' . $row['id'] . ' data-asin=' . $row['asin'] . $data_parent_id . '>
                        <td><i class="fa fa-flag" title="' . $row['sync_status']['text'] . '"></i></td>
                        <td><img src="' . $row['thumb'] . '" alt="' . $row['title'] . '" /> ' . $text_asin . $row['asin'] . ' / ' . $text_id . $row['id'] . $childs_btn . '</td>
                        <td><a href="' . $row['link_edit'] . '" target="_blank">' . $row['title'] . '</a></td>
                        <td><a href="#" title="' . $row['sync_last_status'] . '">' . $text_syncs_nb . '</a>' . $text_syncs_last_status . '</td>
                        <td>' . $row['sync_data'] . '</td>
                        <td class="wwcAmzAff-sync-now"><button>' . $text_sync_now . '</button></td>
                    </tr>
                ';
            } else {
            $ret = '
                    <tr class="wwcAmzAff-sync-table-row' . $tr_css . '" data-id=' . $row['id'] . ' data-asin=' . $row['asin'] . $data_parent_id . '>
                        <td><span>' . $row['id'] . '</span></td>
                        <td><img src="' . $row['thumb'] . '" alt="' . $row['title'] . '" /> ' . $text_asin . $row['asin'] . $childs_btn . '</td>
                        <td><a href="' . $row['link_edit'] . '" target="_blank">' . $row['title'] . '</a></td>
                        <td>' . $text_hits . '</td>
                        <td>' . $text_added_to_cart . '</td>
                        <td>' . $text_redirected_to_amazon . '</td>
                        <td>' . $row['add_data'] . '</td>
                    </tr>
                ';
            }
            return $ret;
        }

        
        /**
         * Get Thumbnails / Thumbnail - based on WP functionality
         */
        private function get_thumb_src_default() {
            return $this->module_folder . 'assets/prod-thumb-default.png';
        }

        private function get_thumbs($size='thumbnail') {
            global $wpdb;
            
            // get post & associated thumbnails id
            $sql = "select p.ID, pm.meta_value from $wpdb->posts as p left join $wpdb->postmeta as pm on p.ID = pm.post_id where 1=1 and pm.meta_key = '_thumbnail_id' and !isnull(pm.meta_id) order by p.ID;";
            $res = $wpdb->get_results( $sql, OBJECT_K );
            if ( empty($res) ) return array();
            
            // get unique thumbnails id
            $sql_thumb = "select distinct(pm.meta_value) from $wpdb->posts as p left join $wpdb->postmeta as pm on p.ID = pm.post_id where 1=1 and pm.meta_key = '_thumbnail_id' and !isnull(pm.meta_id) order by p.ID;";
            $res_thumb = $wpdb->get_results( $sql_thumb, OBJECT_K );
            $thumbsId = array_keys($res_thumb);

            // get meta fields for thumbnails
            $thumb2meta = array('_wp_attachment_metadata' => array(), '_wp_attached_file' => array());
            foreach (array_chunk($thumbsId, self::$sql_chunk_limit, true) as $current) {

                $currentP = implode(',', array_map(array($this->the_plugin, 'prepareForInList'), $current));

                $sql_getmeta = "select p.ID, pm.meta_value from $wpdb->posts as p left join $wpdb->postmeta as pm on p.ID = pm.post_id where 1=1 and pm.meta_key = '_wp_attachment_metadata' and !isnull(pm.meta_id) and pm.post_id IN ($currentP) order by p.ID;";
                $res_getmeta = $wpdb->get_results( $sql_getmeta, OBJECT_K );
                //array_replace($prods2asin, $res_getmeta);
                $thumb2meta['_wp_attachment_metadata'] = $thumb2meta['_wp_attachment_metadata'] + $res_getmeta;
                
                $sql_getmeta = "select p.ID, pm.meta_value, p.guid from $wpdb->posts as p left join $wpdb->postmeta as pm on p.ID = pm.post_id where 1=1 and pm.meta_key = '_wp_attached_file' and !isnull(pm.meta_id) and pm.post_id IN ($currentP) order by p.ID;";
                $res_getmeta = $wpdb->get_results( $sql_getmeta, OBJECT_K );
                //array_replace($prods2asin, $res_getmeta);
                $thumb2meta['_wp_attached_file'] = $thumb2meta['_wp_attached_file'] + $res_getmeta;
            }
 
            $default_meta = array(
                'uploads'       => wp_upload_dir(), // cache this wp function!
            );
            $thumbs = array();
            foreach ($thumbsId as $key) {

                $meta = array_merge($default_meta, array());
                $meta['file'] = isset($thumb2meta['_wp_attached_file']["$key"])
                    ? $thumb2meta['_wp_attached_file']["$key"] : '';
  
                $meta['sizes'] = isset($thumb2meta['_wp_attachment_metadata']["$key"]->meta_value)
                    ? $thumb2meta['_wp_attachment_metadata']["$key"]->meta_value : '';
                if ( !empty($meta['sizes']) ) {
                    $meta['sizes'] = maybe_unserialize($meta['sizes']);
                }
                
                $thumbs["$key"] = $this->get_thumb_src( $meta, 'shop_thumbnail' );
                $thumbs["$key"] = isset($thumbs["$key"][0]) ? $thumbs["$key"][0] : '';
                $thumbs["$key"] = !empty($thumbs["$key"]) ? $thumbs["$key"] : $this->get_thumb_src_default();
            }
   
            $post2thumb = array();
            foreach ( $res as $key => $val ) {
                $thumb_id = $val->meta_value;
                $post2thumb["$key"] = isset($thumbs["$thumb_id"]) && !empty($thumbs["$thumb_id"])
                    ? $thumbs["$thumb_id"] : $this->get_thumb_src_default();
            }
            return $post2thumb;
        }

        private function get_thumb( $meta, $size='medium', $pms=array() ) {
            $image = $this->get_thumb_src( $meta, $size );
            
            $html = '';
            if ( $image ) {
                list($src, $width, $height) = $image;
                $hwstring = image_hwstring($width, $height);
                $size_class = $size;
                if ( is_array( $size_class ) ) {
                    $size_class = join( 'x', $size_class );
                }
                //$attachment = get_post($attachment_id);
                $default_attr = array(
                    'src'   => $src,
                    'class' => "attachment-$size_class",
                    'alt'   => isset($pms['alt']) ? $pms['alt'] : "$size_class", //trim(strip_tags( get_post_meta($attachment_id, '_wp_attachment_image_alt', true) )), // Use Alt field first
                );
                //if ( empty($default_attr['alt']) )
                //    $default_attr['alt'] = trim(strip_tags( $attachment->post_excerpt )); // If not, Use the Caption
                //if ( empty($default_attr['alt']) )
                //    $default_attr['alt'] = trim(strip_tags( $attachment->post_title )); // Finally, use the title
 
                $attr = wp_parse_args($attr, $default_attr);
 
                $attr = array_map( 'esc_attr', $attr );
                $html = rtrim("<img $hwstring");
                foreach ( $attr as $name => $value ) {
                    $html .= " $name=" . '"' . $value . '"';
                }
                $html .= ' />';
            }
            return $html;
        }

        private function get_thumb_src( $meta, $size='medium' ) {
            $img_url = $this->wp_get_attachment_url($meta);
  
            $width = $height = 0;
            $img_url_basename = wp_basename($img_url);
                
            // try for a new style intermediate size
            if ( $intermediate = $this->image_get_intermediate_size($meta, $size) ) {
                $img_url = str_replace($img_url_basename, $intermediate['file'], $img_url);
                $width = $intermediate['width'];
                $height = $intermediate['height'];
            }
            elseif ( $size == 'thumbnail' ) {
                // fall back to the old thumbnail
                $file = isset($meta['file']->meta_value) ? $meta['file']->meta_value : '';

                if ( ($thumb_file = $file) && $info = getimagesize($thumb_file) ) {
                    $img_url = str_replace($img_url_basename, wp_basename($thumb_file), $img_url);
                    $width = $info[0];
                    $height = $info[1];
                }
            }
            
            if ( !$width && !$height && isset( $meta['sizes']['width'], $meta['sizes']['height'] ) ) {
                // any other type: use the real image
                $width = $meta['sizes']['width'];
                $height = $meta['sizes']['height'];
            }
            if ( $img_url) {
                // we have the actual image size, but might need to further constrain it if content_width is narrower
                list( $width, $height ) = image_constrain_size_for_editor( $width, $height, $size );
                return array( $img_url, $width, $height );
            }
            return false;
        }

        private function wp_get_attachment_url( $meta ) {
            $url = '';

            $uploads = $meta['uploads'];
            $file = isset($meta['file']->meta_value) ? $meta['file']->meta_value : '';
            if ( !empty($file) ) {
                // Get upload directory.
                if ( $uploads && false === $uploads['error'] ) {
                    // Check that the upload base exists in the file location.
                    if ( 0 === strpos( $file, $uploads['basedir'] ) ) {
                        // Replace file location with url location.
                        $url = str_replace($uploads['basedir'], $uploads['baseurl'], $file);
                    } elseif ( false !== strpos($file, 'wp-content/uploads') ) {
                        $url = $uploads['baseurl'] . substr( $file, strpos($file, 'wp-content/uploads') + 18 );
                    } else {
                        // It's a newly-uploaded file, therefore $file is relative to the basedir.
                        $url = $uploads['baseurl'] . "/$file";
                    }
                }
            }

            if ( empty($url) ) {
                $url = isset($meta['sizes']->guid) ? $meta['sizes']->guid : '';
            }
            return $url;
        }

        private function image_get_intermediate_size( $meta, $size='thumbnail' ) {
            if ( !is_array( $imagedata = $meta['sizes'] ) )
                return false;

            // get the best one for a specified set of dimensions
            if ( is_array($size) && !empty($imagedata['sizes']) ) {
                foreach ( $imagedata['sizes'] as $_size => $data ) {
                    // already cropped to width or height; so use this size
                    if ( ( $data['width'] == $size[0] && $data['height'] <= $size[1] ) || ( $data['height'] == $size[1] && $data['width'] <= $size[0] ) ) {
                        $file = $data['file'];
                        list($width, $height) = image_constrain_size_for_editor( $data['width'], $data['height'], $size );
                        return compact( 'file', 'width', 'height' );
                    }
                    // add to lookup table: area => size
                    $areas[$data['width'] * $data['height']] = $_size;
                }
                if ( !$size || !empty($areas) ) {
                    // find for the smallest image not smaller than the desired size
                    ksort($areas);
                    foreach ( $areas as $_size ) {
                        $data = $imagedata['sizes'][$_size];
                        if ( $data['width'] >= $size[0] || $data['height'] >= $size[1] ) {
                            // Skip images with unexpectedly divergent aspect ratios (crops)
                            // First, we calculate what size the original image would be if constrained to a box the size of the current image in the loop
                            $maybe_cropped = image_resize_dimensions($imagedata['width'], $imagedata['height'], $data['width'], $data['height'], false );
                            // If the size doesn't match within one pixel, then it is of a different aspect ratio, so we skip it, unless it's the thumbnail size
                            if ( 'thumbnail' != $_size && ( !$maybe_cropped || ( $maybe_cropped[4] != $data['width'] && $maybe_cropped[4] + 1 != $data['width'] ) || ( $maybe_cropped[5] != $data['height'] && $maybe_cropped[5] + 1 != $data['height'] ) ) )
                                continue;
                            // If we're still here, then we're going to use this size
                            $file = $data['file'];
                            list($width, $height) = image_constrain_size_for_editor( $data['width'], $data['height'], $size );
                            return compact( 'file', 'width', 'height' );
                        }
                    }
                }
            }
 
            if ( is_array($size) || empty($size) || empty($imagedata['sizes'][$size]) )
                return false;
 
            $data = $imagedata['sizes'][$size];
            // include the full filesystem path of the intermediate file
            if ( empty($data['path']) && !empty($data['file']) ) {
                $file_url = $this->wp_get_attachment_url($meta);
                $data['path'] = path_join( dirname($imagedata['file']), $data['file'] );
                $data['url'] = path_join( dirname($file_url), $data['file'] );
            }
            return $data;
        }


		/**
         * Ajax requests
         */
		public function ajax_request()
		{
			global $wpdb;
			$request = array(
                'action'                        => isset($_REQUEST['subaction']) ? $_REQUEST['subaction'] : '',
                'module'                        => isset($_REQUEST['module']) ? $_REQUEST['module'] : 'synchronization',
                'sync_fields'                   => isset($_REQUEST['sync_fields']) ? $_REQUEST['sync_fields'] : array(),
                'sync_recurrence'               => isset($_REQUEST['sync_recurrence']) ? $_REQUEST['sync_recurrence'] : '',
                'sync_hour_start'               => isset($_REQUEST['sync_hour_start']) ? $_REQUEST['sync_hour_start'] : '',
                'sync_products_per_request'     => isset($_REQUEST['sync_products_per_request']) ? (int) $_REQUEST['sync_products_per_request'] : 10,
                
                'sync_stop_reload'   => isset($_REQUEST['sync_stop_reload']) ? (int) $_REQUEST['sync_stop_reload'] : 0,
                
                'id'                => isset($_REQUEST['id']) ? (int)$_REQUEST['id'] : 0,
                'asin'              => isset($_REQUEST['asin']) ? (int)$_REQUEST['asin'] : '',
			);
            extract($request);
            
            $ret = array(
                'status'        => 'invalid',
                'msg'           => '<div class="wwcAmzAff-sync-settings-msg wwcAmzAff-message wwcAmzAff-error">' . __('Invalid action!', $this->the_plugin->localizationName) . '</div>',
            );
            
            if ( empty($action) || !in_array($action, array('save_settings', 'load_products', 'sync_prod', 'auto_reload')) ) {
                die(json_encode($ret));
            }
   
            if ( $action == 'save_settings' ) {
                if ( !empty($request['sync_fields']) ) {
                    $request['sync_fields'] = array_keys($request['sync_fields']);
                }
                
                $request = array_diff_key($request, array_fill_keys(array('action', 'id', 'asin'), 1));
                update_option($this->alias . '_sync', (array) $request);
                
                $ret = array_merge($ret, array(
                    'status'    => 'valid',
                    'msg'       => '<div class="wwcAmzAff-sync-settings-msg wwcAmzAff-message wwcAmzAff-success">' . __('Sync settings saved successfully.', $this->the_plugin->localizationName) . '</div>',
                ));
 
            } else if ( $action == 'load_products' ) {
                
                $productsList = $this->get_products( $module );

                $ret = array_merge($ret, array(
                    'status'    => 'valid',
                    'msg'       => '',
                    'html'      => implode(PHP_EOL, $productsList['html']),
                    'nb'        => $productsList['nb'],
                    'nbv'       => $productsList['nbv'],
                ));

            } else if ( $action == 'sync_prod' ) {
                if ( empty($asin) ) {
                    $asin = get_post_meta($id, '_amzASIN', true);
                }

                // sync product!
                require_once( $this->the_plugin->cfg['paths']['plugin_dir_path'] . '/modules/synchronization/init.php' );
                $sync = new wwcAmazonSyncronize( $this->the_plugin );
                $syncStat = $sync->updateTheProduct( $asin, 'return' );


                $last_date = get_post_meta($id, '_amzaff_sync_last_date', true);
                $last_status = get_post_meta($id, '_amzaff_sync_last_status_msg', true);
                $last_status = !empty($last_status) ? strtoupper('last sync status: ' . $last_status) : '';
                //$last_status = !empty($last_status) ? '<i class="fa fa-comment-o" title="' . $last_status . '"></i>' : '';

                $ret = array_merge($ret, $syncStat, array(
                    'status'            => 'valid',
                    'sync_hits'         => (int) get_post_meta($id, '_amzaff_sync_hits', true),
                    'sync_last_date'    => $this->the_plugin->last_update_date('true', $last_date),
                    'sync_last_status'  => $last_status,
                ));

            } else if ( $action == 'auto_reload' ) {
                $ss = get_option($this->alias . '_sync');
                $ss = $ss !== false ? $ss : array();
                $ss['sync_stop_reload'] = $request['sync_stop_reload'];

                update_option($this->alias . '_sync', $ss);
                
                $ret = array_merge($ret, array(
                    'status'    => 'valid',
                    'msg'       => '',
                ));
 
            }
            die(json_encode($ret));
		}
    }
}

// Initialize the wwcAmzAffTailSyncMonitor class
//$wwcAmzAffTailSyncMonitor = new wwcAmzAffTailSyncMonitor($this->cfg, $module);
$wwcAmzAffTailSyncMonitor = wwcAmzAffTailSyncMonitor::getInstance();